/*
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 */


#include "waltoptablet.h"


#define USB_REQ_GET_REPORT		0x01
#define USB_REQ_SET_REPORT		0x09


static int 
waltoptablet_get_report(struct waltoptablet *waltop, unsigned char type, unsigned char id, void *buf, int size)
{
	return usb_control_msg(waltop->usb_dev, usb_rcvctrlpipe(waltop->usb_dev, 0),
				USB_REQ_GET_REPORT, USB_TYPE_CLASS | USB_RECIP_INTERFACE | USB_DIR_IN,
				(type << 8) + id,	waltop->interface_number,
				buf, size,
				USB_CTRL_SET_TIMEOUT);
}


static int 
waltoptablet_set_report(struct waltoptablet *waltop, unsigned char type, unsigned char id, void *buf, int size)
{
	return usb_control_msg(waltop->usb_dev, usb_sndctrlpipe(waltop->usb_dev, 0),
				USB_REQ_SET_REPORT, USB_TYPE_CLASS | USB_RECIP_INTERFACE |USB_DIR_OUT,
				(type << 8) + id,	waltop->interface_number,
				buf, size,
				USB_CTRL_SET_TIMEOUT);
}

/* command function */
int 
waltoptablet_getMaxX(struct waltoptablet* waltop)
{
	unsigned char id[]={2,1,0};
	unsigned char *cmd;
	int size = sizeof(unsigned char)*3;
	int retval, cx;


	cmd = kmalloc(size, GFP_KERNEL);
	if(!cmd){
		return -ENOMEM;
	}

	memcpy(cmd, id, size);

	msleep(100);
	if((retval=waltoptablet_set_report(waltop, 3, 2, cmd, size))!=size){
		printk(KERN_INFO "%s:%d - error:%d\n", __FILE__, __LINE__, retval);
		kfree(cmd);
		return retval<0?retval:0;
	}

	msleep(100);
	if((retval = waltoptablet_get_report(waltop, 3, 2, cmd, size))!=size){
		printk(KERN_INFO "%s:%d - error:%d\n", __FILE__, __LINE__, retval);			
		kfree(cmd);
		return retval<0?retval:0;
	}
	
	cx = cmd[1] + cmd[2] * 256;
	kfree(cmd);
	return cx;
}


int 
waltoptablet_getMaxY(struct waltoptablet* waltop)
{
	unsigned char id[]={2,1,1};
	unsigned char *cmd;
	int size = sizeof(unsigned char)*3;
	int retval, cy;


	cmd = kmalloc(size, GFP_KERNEL);
	if(!cmd){
		return -ENOMEM;
	}

	memcpy(cmd, id, size);

	msleep(100);
	if((retval = waltoptablet_set_report(waltop, 3, 2, cmd, size))!=size){
		printk(KERN_INFO "%s:%d - error:%d\n", __FILE__, __LINE__, retval);
		kfree(cmd);
		return retval<0?retval:0;
	}

	msleep(100);
	if((retval = waltoptablet_get_report(waltop, 3, 2, cmd, size))!=size){
		printk(KERN_INFO "%s:%d - error:%d\n", __FILE__, __LINE__, retval);
		kfree(cmd);
		return retval<0?retval:0;
	}

	cy = cmd[1] + cmd[2] * 256;
	kfree(cmd);
	return cy;	
}


int 
waltoptablet_getMaxPressure(struct waltoptablet* waltop)
{
	unsigned char id[]={2,8,0};
	unsigned char *cmd;
	int size = sizeof(unsigned char)*3;
	int retval, cz;


	cmd = kmalloc(size, GFP_KERNEL);
	if(!cmd){
		return -ENOMEM;
	}

	memcpy(cmd, id, size);

	msleep(100);
	if((retval= waltoptablet_set_report(waltop, 3, 2, cmd, size))!= size){
		printk(KERN_INFO "%s:%d - error:%d\n", __FILE__, __LINE__, retval);
		kfree(cmd);
		return retval<0?retval:0;	
	}

	msleep(100);
	if((retval = waltoptablet_get_report(waltop, 3, 2, cmd, size))!=size){
		printk(KERN_INFO "%s:%d - error:%d\n", __FILE__, __LINE__, retval);
		kfree(cmd);
		return retval<0?retval:0;	
	}
	
	cz = cmd[1] + cmd[2] * 256;
	kfree(cmd);
	return cz;
}


bool 
waltoptablet_setOperationMode_default(struct waltoptablet* waltop)
{
	unsigned char id[]={2,16,0};
	unsigned char *cmd;
	int size = sizeof(unsigned char)*3;
	int retval;


	cmd = kmalloc(size, GFP_KERNEL);
	if(!cmd){
		return false;
	}

	memcpy(cmd, id, size);

	msleep(100);
	if((retval= waltoptablet_set_report(waltop, 3, 2, cmd, size))!= size){
		printk(KERN_ERR "%s:%d - error:%d\n", __FILE__, __LINE__, retval);
		kfree(cmd);
		return false;	
	}
	kfree(cmd);
	return true;
}


bool 
waltoptablet_setOperationMode_tablet(struct waltoptablet* waltop)
{
	unsigned char id[]={2,16,1};
	unsigned char *cmd;
	int size = sizeof(unsigned char)*3;
	int retval;


	cmd = kmalloc(size, GFP_KERNEL);
	if(!cmd){
		return false;
	}

	memcpy(cmd, id, size);

	msleep(100);
	if((retval= waltoptablet_set_report(waltop, 3, 2, cmd, size))!= size){
		printk(KERN_ERR "%s:%d - error:%d\n", __FILE__, __LINE__, retval);
		kfree(cmd);
		return false;	
	}
	kfree(cmd);
	return true;	
}


bool 
waltoptablet_setOperationMode_ink(struct waltoptablet* waltop)
{
	unsigned char id[]={2,16,2};
	unsigned char *cmd;
	int size = sizeof(unsigned char)*3;
	int retval;


	cmd = kmalloc(size, GFP_KERNEL);
	if(!cmd){
		return false;
	}

	memcpy(cmd, id, size);

	msleep(100);
	if((retval= waltoptablet_set_report(waltop, 3, 2, cmd, size))!= size){
		printk(KERN_ERR "%s:%d - error:%d\n", __FILE__, __LINE__, retval);
		kfree(cmd);
		return false;	
	}
	kfree(cmd);
	return true;	
}
