/*
 * based on linuxwacom-0.8.0-3
 */


#include "waltop-tablet.h"
#include "waltop-usb.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <memory.h>
#include <fcntl.h>
#include <unistd.h>
#include <assert.h>
#include <stdarg.h>

#include <linux/input.h>



/*****************************************************************************
** Tablet function
*****************************************************************************/

WALTOPTABLET WaltopOpenTablet(const char* pszDevice)
{
	int fd, e;
	WALTOPTABLET hTablet = NULL;

	/* open device for read/write access */
	fd = open(pszDevice,O_RDWR);
	if (fd < 0) 
	{
		e = errno;
		fprintf(stderr, "Failed to open %s: %s", pszDevice, strerror(errno));
		errno = e;
		return NULL;
	}

	/* configure usb */
	hTablet = OpenUSBTablet(fd);
	if (!hTablet) { e=errno; close(fd); errno=e; return NULL; }

	return hTablet;
}


int WaltopCopyState(WALTOPSTATE* pDest, WALTOPSTATE* pSrc)
{
	unsigned int uCnt;
	if (!pSrc || !pDest || !pSrc->uValueCnt || !pDest->uValueCnt)
		{ errno=EINVAL; return 1; }

	/* copy valid bits over */
	pDest->uValid = pSrc->uValid;

	/* determine how many values to transfer */
	uCnt = (pDest->uValueCnt < pSrc->uValueCnt) ?
			pDest->uValueCnt : pSrc->uValueCnt;

	/* copy them over */
	memcpy(pDest->values,pSrc->values,uCnt*sizeof(WALTOPVALUE));

	return 0;
}


/*****************************************************************************
** Virtual Functions
*****************************************************************************/

void WaltopCloseTablet(WALTOPTABLET hTablet)
{
	WALTOPTABLET_PRIV* pTablet = (WALTOPTABLET_PRIV*)hTablet;
	if (!pTablet || !pTablet->Close) return;
	pTablet->Close(pTablet);
}


int WaltopGetCapabilities(WALTOPTABLET hTablet)
{
	WALTOPTABLET_PRIV* pTablet = (WALTOPTABLET_PRIV*)hTablet;
	if (!pTablet || !pTablet->GetCaps) { errno=EBADF; return 0; }
	return pTablet->GetCaps(pTablet);
}

int WaltopGetState(WALTOPTABLET hTablet, WALTOPSTATE* pState)
{
	WALTOPTABLET_PRIV* pTablet = (WALTOPTABLET_PRIV*)hTablet;
	if (!pTablet || !pTablet->GetState) { errno=EBADF; return 0; }
	return pTablet->GetState(pTablet,pState);
}

int WaltopGetFileDescriptor(WALTOPTABLET hTablet)
{
	WALTOPTABLET_PRIV* pTablet = (WALTOPTABLET_PRIV*)hTablet;
	if (!pTablet || !pTablet->GetFD) { errno=EBADF; return -1; }
	return pTablet->GetFD(pTablet);
}

void WaltopSetFileDescriptor(WALTOPTABLET hTablet, int fd)
{
	WALTOPTABLET_PRIV* pTablet = (WALTOPTABLET_PRIV*)hTablet;
	if (!pTablet || !pTablet->SetFD) { errno=EBADF; return; }
	pTablet->SetFD(pTablet, fd);
}

int WaltopReadRaw(WALTOPTABLET hTablet, unsigned char* puchData,
		unsigned int uSize)
{
	WALTOPTABLET_PRIV* pTablet = (WALTOPTABLET_PRIV*)hTablet;
	if (!pTablet || !pTablet->ReadRaw) { errno=EBADF; return 0; }
	return pTablet->ReadRaw(pTablet,puchData,uSize);
}

int WaltopParseData(WALTOPTABLET hTablet, const unsigned char* puchData,
		unsigned int uLength, WALTOPSTATE* pState)
{
	WALTOPTABLET_PRIV* pTablet = (WALTOPTABLET_PRIV*)hTablet;
	if (!pTablet || !pTablet->ParseData) { errno=EBADF; return 0; }
	return pTablet->ParseData(pTablet,puchData,uLength,pState);
}


