/*
 *
 * based on wacomlinux-0.8.0-3 
 *
 */

#ifndef WALTOPTABLET_UTIL_H
#define WALTOPTABLET_UTIL_H


#include <stdarg.h>
#include <sys/time.h>

#define WALTOPTOOLTYPE_NONE      0x00
#define WALTOPTOOLTYPE_PEN        0x01
#define WALTOPTOOLTYPE_MAX       0x02


#define WALTOPBUTTON_LEFT        		0
#define WALTOPBUTTON_MIDDLE      	1
#define WALTOPBUTTON_RIGHT       	2
#define WALTOPBUTTON_EXTRA       	3
#define WALTOPBUTTON_SIDE        		4
#define WALTOPBUTTON_TOUCH       	5
#define WALTOPBUTTON_STYLUS      	6
#define WALTOPBUTTON_MAX         	7


#define WALTOPFIELD_TOOLTYPE    	0
#define WALTOPFIELD_PROXIMITY    	1
#define WALTOPFIELD_BUTTONS      	2
#define WALTOPFIELD_POSITION_X   	3
#define WALTOPFIELD_POSITION_Y   	4
#define WALTOPFIELD_PRESSURE	      5
#define WALTOPFIELD_MAX          		6


typedef struct
{
	int nValue;
	int nMin;
	int nMax;
	int nReserved;
} WALTOPVALUE;

typedef struct
{
	unsigned int uValueCnt;     /* This MUST be set to WALTOPFIELD_MAX. */
	unsigned int uValid;        /* Bit mask of WALTOPFIELD_xxx bits. */
	WALTOPVALUE values[WALTOPFIELD_MAX];
} WALTOPSTATE;

#define WALTOPSTATE_INIT { WALTOPFIELD_MAX }


/*****************************************************************************
** Public structures
*****************************************************************************/

typedef struct { int __unused; } *WALTOPTABLET;


/*****************************************************************************
** Public API
*****************************************************************************/

	WALTOPTABLET WaltopOpenTablet(const char* pszDevice);
	/* Returns tablet handle on success, NULL otherwise.
	 * pszDevice is pathname to device.  Model may be NULL; if any model
	 * parameters are 0, detection is automatic. */

	int WaltopCopyState(WALTOPSTATE* pDest, WALTOPSTATE* pSrc);
	/* Returns 0 on success.  Copies tablet state structures.
	 * Source and destination structures must be properly initialized,
	 * particularly the uValueCnt field must be set WACOMFIELD_MAX.
	 * Returns 0 on success. */

	void WaltopCloseTablet(WALTOPTABLET hTablet);
	/* Releases all resource associated with tablet and closes device. */

	int WaltopGetCapabilities(WALTOPTABLET hTablet);
	/* Returns bitmask of valid fields.  Use (1 << WACOMFIELD_xxx) to
	 * translate field identifiers to bit positions.  This API will only
	 * support 32 capabilities. */

	int WaltopGetState(WALTOPTABLET hTablet, WALTOPSTATE* pState);
	/* Returns 0 on success.  Tablet state is copied to specified structure.
	 * Data is only a snapshot of the device and may not accurately reflect
	 * the position of any specific tool. This is particularly true of
	 * multi-tool mode tablets.  NOTE: pState must point to an initialized
	 * structure; the uValueCnt field must be correctly set to
	 * WALTOPFIELD_MAX. */

	int WaltopGetFileDescriptor(WALTOPTABLET hTablet);
	/* Returns the file descriptor of the tablet or -1 on error. */

	void WaltopSetFileDescriptor(WALTOPTABLET hTablet, int fd);
	/* Returns the file descriptor of the tablet or -1 on error. */

	int WaltopReadRaw(WALTOPTABLET hTablet, unsigned char* puchData,
			unsigned int uSize);
	/* Returns number of bytes read, typically a single packet.  Call will
	 * block.  uSize should be the maximum size of the given data buffer. */

	int WaltopParseData(WALTOPTABLET hTablet, const unsigned char* puchData,
			unsigned int uLength, WALTOPSTATE* pState);
	/* Updates the tablet state from a given packet.  If pState is specified,
	 * the tablet state is copied to the given structure.  This structure
	 * must be correctly initialized; the uValueCnt member must be set to
	 * WALTOPFIELD_MAX. Returns 0 on success. */

/*****************************************************************************
** Private structures
*****************************************************************************/

typedef struct _WALTOPTABLET_PRIV WALTOPTABLET_PRIV;

struct _WALTOPTABLET_PRIV
{
	void (*Close)(WALTOPTABLET_PRIV* pTablet);
	
	int (*GetCaps)(WALTOPTABLET_PRIV* pTablet);
	int (*GetState)(WALTOPTABLET_PRIV* pTablet, WALTOPSTATE* pState);
	int (*GetFD)(WALTOPTABLET_PRIV* pTablet);
	void (*SetFD)(WALTOPTABLET_PRIV* pTablet, int fd);
	int (*ReadRaw)(WALTOPTABLET_PRIV* pTablet, unsigned char* puchData,
			unsigned int uSize);
	int (*ParseData)(WALTOPTABLET_PRIV* pTablet, const unsigned char* puchData,
			unsigned int uLength, WALTOPSTATE* pState);
};

#endif

